package com.jpattern.orm.persistor.reflection;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;

import com.jpattern.orm.exception.OrmConfigurationException;

/**
 * 
 * Get the value of a field directly accessing his value
 * 
 * @author Francesco Cina'
 *
 * Mar 31, 2012
 */
public class FieldSetManipulator<BEAN, P> extends SetManipulator<BEAN, P> {

	private final Field field;

	public FieldSetManipulator(final Field field) {
		this.field = field;
		field.setAccessible(true);

		if (Modifier.isFinal(field.getModifiers()) ) {
			throw new OrmConfigurationException("Field [" + field.getName() + "] of class [" + field.getDeclaringClass() + "] is marked FINAL. His value cannot be managed by JPOrm. Please remove the 'final' modifier.");
		}

	}

	@Override
	public void setValue(final BEAN bean, final P value) throws IllegalArgumentException, IllegalAccessException {
		this.field.set(bean, value);
	}

}
