package com.jpattern.orm.query.find;

import java.util.List;

import com.jpattern.orm.exception.OrmException;
import com.jpattern.orm.exception.OrmNotUniqueResultException;
import com.jpattern.orm.query.LockMode;
import com.jpattern.orm.query.OrmRowMapper;
import com.jpattern.orm.query.clause.OrmOrderBy;

/**
 * 
 * @author ufo
 *
 * @param <BEAN>
 */
public class FindOrderByImpl<BEAN> extends OrmOrderBy<FindOrderBy<BEAN>> implements FindOrderBy<BEAN> {

	private final FindQuery<BEAN> findQuery;

	@Override
	public String renderSql() {
		return this.findQuery.renderSql();
	}

	@Override
	public void renderSql(final StringBuilder stringBuilder) {
		this.findQuery.renderSql(stringBuilder);
	}

	@Override
	public void appendValues(final List<Object> values) {
		this.findQuery.appendValues(values);
	}

	public FindOrderByImpl(final FindQuery<BEAN> findQuery) {
		this.findQuery = findQuery;

	}

	@Override
	public FindQuery<BEAN> query() {
		return this.findQuery;
	}

	@Override
	public FindWhere<BEAN> where() throws OrmException {
		return this.findQuery.where();
	}

	@Override
	protected FindOrderBy<BEAN> orderBy() throws OrmException {
		return this;
	}

	@Override
	public List<BEAN> findList() throws OrmException {
		return this.findQuery.findList();
	}

	@Override
	public long findRowCount() throws OrmException {
		return this.findQuery.findRowCount();
	}

	@Override
	public BEAN findUnique() throws OrmException, OrmNotUniqueResultException {
		return this.findQuery.findUnique();
	}

	@Override
	public void find(final OrmRowMapper<BEAN> srr) throws OrmException {
		this.findQuery.find(srr);
	}

	@Override
	public String getGeneratedRowCountSql() throws OrmException {
		return this.findQuery.getGeneratedRowCountSql();
	}

	@Override
	public FindQuery<BEAN> setDistinct(final boolean distinct) throws OrmException {
		return this.findQuery.setDistinct(distinct);
	}

	@Override
	public boolean isDistinct() throws OrmException {
		return this.findQuery.isDistinct();
	}

	@Override
	public FindQuery<BEAN> setLockMode(final LockMode lockMode) {
		return this.findQuery.setLockMode(lockMode);
	}

	@Override
	public LockMode getLockMode() {
		return this.findQuery.getLockMode();
	}

	@Override
	public int getMaxRows() throws OrmException {
		return this.findQuery.getMaxRows();
	}

	@Override
	public FindQuery<BEAN> setMaxRows(final int maxRows) throws OrmException {
		return this.findQuery.setMaxRows(maxRows);
	}

	@Override
	public FindQuery<BEAN> setQueryTimeout(final int queryTimeout) {
		return this.findQuery.setQueryTimeout(queryTimeout);
	}

	@Override
	public int getQueryTimeout() {
		return this.findQuery.getQueryTimeout();
	}

}
